"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _WebsocketClient_base = _interopRequireDefault(require("./WebsocketClient_base"));

var _isomorphicWs = _interopRequireDefault(require("isomorphic-ws"));

var _logger = require("../logger");

var _exceptions = require("../exceptions");

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const config = require("../config");

class WebsocketClient extends _WebsocketClient_base.default {
  connect() {
    return new Promise((resolve, reject) => {
      try {
        this.setPromiseHandlers(resolve, reject);
        this._connection = new _isomorphicWs.default(this.url, [], {
          headers: {
            "User-Agent": `RookoutAgent/${config.VersionConfiguration.VERSION}+${config.VersionConfiguration.COMMIT}`,
            "X-Rookout-Token": this.token
          },
          agent: this.buildProxy(),
          perMessageDeflate: false,
          rejectUnauthorized: !config.AgentComConfiguration.SKIP_SSL_CERTIFICATE_VERIFICATION
        });
        this.setConnectTimeout();
        this.setResponseListeners();
      } catch (e) {
        reject(e);
      }
    });
  }

  sendInner(message, timeout, resolve, reject) {
    this._connection.send(message, {
      binary: true
    }, err => {
      if (err) {
        reject(new _exceptions.RookSendFailedError());
      } else {
        resolve(); // send was successful
      }

      clearTimeout(timeout);
    });
  }

  terminateConnection() {
    if (this._connection) {
      this._connection.terminate();
    }

    this._connection = null;
  }

  close() {
    this.terminateConnection();
    this.clearPingTimeout();
    this.clearPingInterval();
    this.clearConnectTimeout();
  }

  buildProxy() {
    if (this.proxy === null) {
      return false;
    }

    const HttpsProxyAgent = require('https-proxy-agent');

    _logger.logger.debug("Connecting via proxy: %s", this.proxy);

    if (this.proxy.startsWith("http")) {
      return HttpsProxyAgent(this.proxy);
    } else {
      return HttpsProxyAgent("http://" + this.proxy);
    }
  }

  setResponseListeners() {
    // Heartbeat runs on every ping and pong, as well as once on a new connection.
    this._connection.on("open", () => {
      try {
        _logger.logger.debug("Connection established");

        this.clearConnectTimeout(); // The socket does not exist before this point except internally - we can't access it.
        // Upon a connection timeout (currently 2 seconds), the socket will be terminated.

        if (this._connection) {
          (0, _utils.nodeUnref)(this._connection._socket);
        } // Set up pinging


        this.setPingInterval();
        this.heartbeat();
        this.returnConnected();
      } catch (e) {}
    });

    this._connection.on("ping", () => {
      try {
        this.heartbeat();
      } catch (e) {}
    });

    this._connection.on("pong", () => {
      try {
        this.heartbeat();
      } catch (e) {}
    });

    this._connection.on("close", () => {
      try {
        this.returnError('Connection closed');
      } catch (e) {}
    });

    this._connection.on("message", msg => {
      try {
        this.returnMessage(msg);
      } catch (e) {}
    });

    this._connection.on("error", err => {
      try {
        this.returnError(err);
      } catch (e) {}
    }); // Unexpected HTTP responses


    this._connection.on("unexpected-response", (req, resp) => {
      try {
        this.returnError({
          statusCode: resp.statusCode
        });
      } catch (e) {}
    });
  }

}

exports.default = WebsocketClient;