"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cleanupAgentInfoPromise = cleanupAgentInfoPromise;
exports.collect = collect;
exports.pack_agent_info = pack_agent_info;

var _information_base = _interopRequireDefault(require("./information_base"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const agent_info_pb = require("../protobuf/agent_info_pb");

const config = require('../config');

class PlatformInformation extends _information_base.default {
  _getCollectors() {
    return {
      "platform": "browser",
      "version": "v1.0.0"
    };
  }

}

class NetworkInformation extends _information_base.default {
  _getCollectors() {
    return {
      "ip_addr": "network.ip_addr",
      "network": "network.network"
    };
  }

}

class SystemInformation extends _information_base.default {
  _getCollectors() {
    return {
      "hostname": "os.hostname",
      "os": navigator.userAgent,
      "os_version": "os.release",
      "distro": "this._getDistro",
      "arch": "os.arch"
    };
  }

}

class VersionInformation extends _information_base.default {
  _getCollectors() {
    return {
      "version": "0.115.0",
      "commit": "60c6f7d1ee70ac89d44b1b45e7f8aef4b8d8ba8c"
    };
  }

}

class SCMInformation extends _information_base.default {
  _getCollectors() {
    return {
      "commit": this._get_commit.bind(this),
      "origin": this._get_origin.bind(this),
      "sources": this._get_sources.bind(this)
    };
  }

  _get_commit() {
    return config.GitConfiguration.GIT_COMMIT || "";
  }

  _get_origin() {
    return config.GitConfiguration.GIT_ORIGIN || "";
  }

  _get_sources() {
    return config.GitConfiguration.SOURCES || {};
  }

}

class AgentInformation extends _information_base.default {
  _getCollectors() {
    return {
      "version": () => new VersionInformation().collect(),
      "network": () => new NetworkInformation().collect(),
      "system": () => new SystemInformation().collect(),
      "platform": () => new PlatformInformation().collect(),
      "scm": () => new SCMInformation().collect(),
      "executable": window.location.origin + window.location.pathname,
      "command_arguments": window.location.search,
      "process_id": 0
    };
  }

}

let agentInfoPromise = null;

async function collect(debug) {
  if (!agentInfoPromise) {
    agentInfoPromise = new AgentInformation().collect(debug);
  }

  return agentInfoPromise;
}

function cleanupAgentInfoPromise() {
  if (agentInfoPromise) {
    agentInfoPromise = null;
  }
}

function pack_agent_info(info) {
  let packed_info = new agent_info_pb.AgentInformation();
  packed_info.setAgentId(info.agent_id);
  let version_info = new agent_info_pb.VersionInformation();
  version_info.setVersion(info.version.version);
  version_info.setCommit(info.version.commit);
  packed_info.setVersion(version_info);
  let network_info = new agent_info_pb.NetworkInformation();
  network_info.setIpAddr(info.network.ip_addr);
  network_info.setNetwork(info.network.network);
  packed_info.setNetwork(network_info);
  let system_info = new agent_info_pb.SystemInformation();
  system_info.setHostname(info.system.hostname);
  system_info.setOs(info.system.os);
  system_info.setOsVersion(info.system.os_version);
  system_info.setDistro(info.system.distro);
  system_info.setArch(info.system.arch);
  packed_info.setSystem(system_info);
  let platform_info = new agent_info_pb.PlatformInformation();
  platform_info.setPlatform(info.platform.platform);
  platform_info.setVersion(info.platform.version);
  packed_info.setPlatform(platform_info);
  let scm_info = new agent_info_pb.SCMInformation();
  scm_info.setCommit(info.scm.commit);
  scm_info.setOrigin(info.scm.origin);
  packed_info.setScm(scm_info);
  packed_info.setExecutable(info.executable);
  packed_info.setCommandArgumentsList(info.command_arguments);
  packed_info.setProcessId(info.process_id);
  let labels_map = packed_info.getLabelsMap();

  if (info.hasOwnProperty('labels')) {
    Object.keys(info.labels).forEach(key => {
      //set only labels that have values
      if (info.labels[key]) {
        labels_map.set(key, info.labels[key]);
      }
    });
  }

  if (info.debug) {
    labels_map.set("rookout_debug", "on");
  }

  packed_info.setTagsList(info.tags);
  return packed_info;
}